'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _QueryBuilder = require('./QueryBuilder');

var _QueryBuilder2 = _interopRequireDefault(_QueryBuilder);

var _config = require('./config');

var _bunchOfExceptions = require('bunch-of-exceptions');

var _ResponseHandler = require('./ResponseHandler');

var _ResponseHandler2 = _interopRequireDefault(_ResponseHandler);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Model = function () {
    function Model() {
        _classCallCheck(this, Model);

        this.queryBuilder = new _QueryBuilder2.default();
        this.selfValidate();
        this.type = this.getNamespace();

        this.owner = null;

        this.fields = this.getFields();
        this.relations = this.getRelations();
        this.dates = this.getDates();
        this.namespace = this.getNamespace();
    }

    // override

    _createClass(Model, [{
        key: 'getFields',
        value: function getFields() {
            throw (0, _bunchOfExceptions.UnimplementedException)("Method getFields() must be implemented in every child class.");
        }
    }, {
        key: 'getDates',
        value: function getDates() {
            return {};
        }
    }, {
        key: 'getRelations',
        value: function getRelations() {
            return {};
        }
    }, {
        key: 'computed',
        value: function computed() {
            return {};
        }
    }, {
        key: 'getNamespace',
        value: function getNamespace() {
            throw (0, _bunchOfExceptions.UnimplementedException)("Method getNamespace() must be implemented in every child class.");
        }
    }, {
        key: 'getBaseUrl',
        value: function getBaseUrl() {
            return _config.REST_CONFIG.base_url;
        }

        /**
         * Make changes or use fetch config before the request
         * @param fetchConfig the config
         * @returns the changed fetch config
         */

    }, {
        key: 'beforeFetch',
        value: function beforeFetch(fetchConfig) {
            return fetchConfig;
        }

        /**
         * Make changes or use fetch response before the jsoning
         * @param fetchResponse the fetch response
         * @returns the changed response
         */

    }, {
        key: 'afterFetch',
        value: function afterFetch(fetchResponse) {
            return fetchResponse;
        }
    }, {
        key: 'toJson',
        value: function toJson() {
            var json = {};

            var fields = this.getFields();
            var relations = this.getRelations();
            var relationsName = Object.keys(relations);
            var dates = this.getDates();
            var datesName = Object.keys(dates);

            for (var prop in this) {
                if (fields.includes(prop)) {
                    json[prop] = this[prop];
                } else if (datesName.includes(prop)) {
                    json[prop] = this[prop].toString();
                    if (json[prop].includes('T')) {
                        json[prop] = json[prop].replace('T', ' ').replace('Z', '');
                    }
                } else if (relationsName.includes(prop)) {
                    if (relations[prop].list) {
                        json[prop] = this[prop].map(function (one) {
                            return one.toJson();
                        });
                    } else {
                        json[prop] = this[prop].toJson();
                    }
                }
            }

            return json;
        }

        /**
         * Force Lazy loading of a relation to the model and returns the loaded relation
         * @param {string} relation the relation to load and to retrieve
         * @returns {Promise<*>} the loaded relation
         */

    }, {
        key: 'forceLazyLoadAndGet',
        value: async function forceLazyLoadAndGet(relation) {
            await this.forceLazyLoad(relation);
            return this[relation];
        }

        /**
         * Lazy loads a relation to the model and returns the loaded relation
         * @param {string} relation the relation to load and to retrieve
         * @returns {Promise<*>} the loaded relation
         */

    }, {
        key: 'lazyLoadAndGet',
        value: async function lazyLoadAndGet(relation) {
            await this.lazyLoad(relation);
            return this[relation];
        }

        /**
         * Lazy loads relations to the model.
         * Does not load already loaded relations.
         * To force the load, use Model#forceLazyLoad() method
         *
         * @param {...string} relations a list of relations to load. The function doesn't reload an already loaded relation.
         * @returns {Promise<*>} formatted loaded relation if single, otherwise returns this.
         */

    }, {
        key: 'lazyLoad',
        value: async function lazyLoad() {
            for (var _len = arguments.length, relations = Array(_len), _key = 0; _key < _len; _key++) {
                relations[_key] = arguments[_key];
            }

            relations = this.filterLoadedRelations.apply(this, _toConsumableArray(relations));
            return this.forceLazyLoad.apply(this, _toConsumableArray(relations));
        }

        /**
         * Lazy loads relations to the model.
         * This function FORCE all relations to be loaded, even the already loaded ones.
         * To avoid the force reload, use Model#lazyLoad() method
         *
         * @param {...string} relations a list of relations to load.
         * @returns {Promise<*>} this.
         */

    }, {
        key: 'forceLazyLoad',
        value: async function forceLazyLoad() {
            for (var _len2 = arguments.length, relations = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
                relations[_key2] = arguments[_key2];
            }

            if (relations.length > 0) {
                try {
                    var _ref;

                    var newThis = await (_ref = new this.constructor()).with.apply(_ref, relations).find(this.id);
                    var _iteratorNormalCompletion = true;
                    var _didIteratorError = false;
                    var _iteratorError = undefined;

                    try {
                        for (var _iterator = relations[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                            var r = _step.value;

                            this[r] = newThis[r];
                            this.relations[r].loaded = true;
                        }
                    } catch (err) {
                        _didIteratorError = true;
                        _iteratorError = err;
                    } finally {
                        try {
                            if (!_iteratorNormalCompletion && _iterator.return) {
                                _iterator.return();
                            }
                        } finally {
                            if (_didIteratorError) {
                                throw _iteratorError;
                            }
                        }
                    }
                } catch (e) {
                    console.error('Lazy loading error. Relation \'' + relations.join("', '") + '\' of \'' + this.constructor.name + '\':');
                    console.error(e);
                    var _iteratorNormalCompletion2 = true;
                    var _didIteratorError2 = false;
                    var _iteratorError2 = undefined;

                    try {
                        for (var _iterator2 = relations[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
                            var _r = _step2.value;

                            this.relations[_r] = [];
                        }
                    } catch (err) {
                        _didIteratorError2 = true;
                        _iteratorError2 = err;
                    } finally {
                        try {
                            if (!_iteratorNormalCompletion2 && _iterator2.return) {
                                _iterator2.return();
                            }
                        } finally {
                            if (_didIteratorError2) {
                                throw _iteratorError2;
                            }
                        }
                    }
                }
            }

            return this;
        }

        /**
         * Filter the relations that have already been loaded.
         * @param {string} relations the model's relations
         * @returns {array} the filtered relations (that haven't been loaded yet)
         */

    }, {
        key: 'filterLoadedRelations',
        value: function filterLoadedRelations() {
            var arr = [];

            for (var _len3 = arguments.length, relations = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
                relations[_key3] = arguments[_key3];
            }

            var _iteratorNormalCompletion3 = true;
            var _didIteratorError3 = false;
            var _iteratorError3 = undefined;

            try {
                for (var _iterator3 = relations[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
                    var r = _step3.value;

                    if (!this.relations[r].loaded) {
                        arr.push(r);
                    }
                }
            } catch (err) {
                _didIteratorError3 = true;
                _iteratorError3 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion3 && _iterator3.return) {
                        _iterator3.return();
                    }
                } finally {
                    if (_didIteratorError3) {
                        throw _iteratorError3;
                    }
                }
            }

            return arr;
        }

        // requests

    }, {
        key: 'request',
        value: async function request(url, config) {
            this.queryBuilder.reset();

            config.headers = {
                "Content-Type": 'application/json'
            };

            if (_config.REST_CONFIG.cross_origin) {
                config.mode = 'cors';
            }

            config = this.beforeFetch(config);

            var response = await fetch(url, config);

            if (!response.ok) {
                if (response.status >= 500) {
                    throw new _bunchOfExceptions.UnreachableServerException('The server returned HTTP code ' + response.status + ' (' + response.statusText + ')');
                }

                if (response.status >= 400) {
                    throw new _bunchOfExceptions.InvalidUrlException('The server returned HTTP code ' + response.status + ' (' + response.statusText + ')');
                }

                throw new _bunchOfExceptions.Exception('Fetch failed with response: ' + response.statusText);
            }

            response = this.afterFetch(response);

            return await response.json();
        }
    }, {
        key: '_getUrlFunctionOfId',
        value: function _getUrlFunctionOfId(id) {
            if (id) {
                return this.queryBuilder.buildUrl(this.namespace, id);
            } else {
                var snakeCase = this.constructor.name.replace(/([A-Z]+)/g, function (x, y) {
                    return "_" + y.toLowerCase();
                }).replace(/^_/, "");

                return this.queryBuilder.buildUrl(snakeCase);
            }
        }
    }, {
        key: 'find',
        value: async function find(id) {
            this.queryBuilder.awaitType = _QueryBuilder.QUERY_AWAIT_SINGLE;

            var url = this._getUrlFunctionOfId(id);

            var response = await this.request(url, {
                method: _config.REST_CONFIG.http_methods.get
            });

            return this.respond(response.data);
        }
    }, {
        key: 'all',
        value: async function all() {
            var url = this.queryBuilder.buildUrl(this.namespace);
            var response = await this.request(url, {
                method: _config.REST_CONFIG.http_methods.get
            });

            return this.respond(response.data);
        }
    }, {
        key: 'paginate',
        value: async function paginate() {
            var perPage = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 10;
            var page = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 1;

            this.queryBuilder.paginate(perPage, page);

            var url = this.queryBuilder.buildUrl(this.namespace);
            var response = await this.request(url, {
                method: _config.REST_CONFIG.http_methods.get
            });

            return this.respond(response.data);
        }
    }, {
        key: 'save',
        value: async function save() {
            this.queryBuilder.awaitType = _QueryBuilder.QUERY_AWAIT_SINGLE;
            if (this.hasOwnProperty('id')) {
                return this.update();
            }

            return this.create();
        }
    }, {
        key: 'create',
        value: async function create() {
            this.queryBuilder.awaitType = _QueryBuilder.QUERY_AWAIT_SINGLE;

            var url = this.queryBuilder.buildUrl(this.namespace);
            var response = await this.request(url, {
                method: _config.REST_CONFIG.http_methods.create,
                body: JSON.stringify(this.toJson())
            });

            return this.respond(response.data);
        }
    }, {
        key: 'update',
        value: async function update() {
            this.queryBuilder.awaitType = _QueryBuilder.QUERY_AWAIT_SINGLE;

            var url = this.queryBuilder.buildUrl(this.namespace, this.id);
            var response = await this.request(url, {
                method: _config.REST_CONFIG.http_methods.update,
                body: JSON.stringify(this.toJson())
            });

            return this.respond(response.data);
        }
    }, {
        key: 'delete',
        value: async function _delete() {
            this.queryBuilder.awaitType = _QueryBuilder.QUERY_AWAIT_SINGLE;

            var url = this.queryBuilder.buildUrl(this.namespace, this.id);
            var response = this.request(url, {
                method: _config.REST_CONFIG.http_methods.delete
            });

            return this.respond(response.data);
        }
    }, {
        key: 'attach',
        value: async function attach(model) {
            var body = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
            var sync = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

            this.queryBuilder.awaitType = _QueryBuilder.QUERY_AWAIT_SINGLE;

            this.relationOfModel(this);
            var url = this.queryBuilder.buildUrl(model.namespace, model.id);
            var queryConfig = {
                method: sync ? _config.REST_CONFIG.http_methods.update : _config.REST_CONFIG.http_methods.create
            };

            if (body) {
                queryConfig.body = JSON.stringify(body);
            }

            var response = await this.request(url, queryConfig);

            return this.respond(response.data);
        }
    }, {
        key: 'sync',
        value: async function sync(model, data) {
            var detaching = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;

            this.queryBuilder.awaitType = _QueryBuilder.QUERY_AWAIT_SINGLE;
            if (!detaching) {
                this.queryBuilder.addCustomParameter(_config.REST_CONFIG.request_keywords.sync_detaching, detaching);
            }
            return this.attach(model, data, true);
        }
    }, {
        key: 'detach',
        value: async function detach(model) {
            this.queryBuilder.awaitType = _QueryBuilder.QUERY_AWAIT_SINGLE;

            this.relationOfModel(this);

            var url = this.queryBuilder.buildUrl(model.namespace, model.id);
            var response = await this.request(url, {
                method: _config.REST_CONFIG.http_methods.delete
            });

            return this.respond(response.data);
        }

        // modify url string

        /**
         * Load model with some relations
         * @param {string} relation the name of the relation
         * @returns {Model} this
         */

    }, {
        key: 'with',
        value: function _with() {
            var _queryBuilder;

            (_queryBuilder = this.queryBuilder).with.apply(_queryBuilder, arguments);

            return this;
        }

        /**
         * Load as a relation of another model <br/>
         * @note See Model#relationOf if you want to use the namespace and the id
         * @param {Model} model the owner
         * @return {Model} this
         */

    }, {
        key: 'relationOfModel',
        value: function relationOfModel(model) {
            return this.relationOf(model.constructor, model.id);
        }

        /**
         * @note See Model#relationOfModel if you have an instance of a Model
         * @param {constructor} constructor the url constructor of the owner. <br/>
         * @example If the url should be'.../api/users/2/posts' => constructor is 'users'
         * @param id the owner's id
         * @return {Model} this
         */

    }, {
        key: 'relationOf',
        value: function relationOf(constructor, id) {
            if (constructor instanceof Model) {
                id = constructor.id;
                constructor = constructor.constructor;
            }

            this.owner = {
                constructor: constructor,
                id: id
            };

            this.queryBuilder.relationOf(new constructor().namespace, id);

            return this;
        }

        /**
         * @note See Model#relationOfModel if you have an instance of a Model
         * @param {constructor} constructor the url namespace of the owner. <br/>
         * @example If the url should be'.../api/users/2/posts' => namespace is 'users'
         * @param id the owner's id
         * @return {Model} this
         */

    }, {
        key: 'of',
        value: function of(constructor, id) {
            return this.relationOf(constructor, id);
        }

        /**
         * Load as a relation of another model <br/>
         * @note See Model#relationOf if you want to use the namespace and the id
         * @param {Model} model the owner
         * @return {Model} this
         */

    }, {
        key: 'ofModel',
        value: function ofModel(model) {
            return this.relationOfModel(model);
        }

        /**
         * Order by a list of fields
         * @param fields the field string, ASC by default. add a '-' ahead to sort in DESC order.
         * EX: model.orderBy('age', '-name') will order by age ASC, and by name DESC
         * @returns {Model} this
         */

    }, {
        key: 'orderBy',
        value: function orderBy() {
            for (var _len4 = arguments.length, fields = Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
                fields[_key4] = arguments[_key4];
            }

            var _iteratorNormalCompletion4 = true;
            var _didIteratorError4 = false;
            var _iteratorError4 = undefined;

            try {
                for (var _iterator4 = fields[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
                    var field = _step4.value;

                    var direction = 'ASC';
                    var column = field;
                    if (column[0] === '-') {
                        direction = 'DESC';
                        column = field.substr(1);
                    }

                    this.queryBuilder.orderBy(column, direction);
                }
            } catch (err) {
                _didIteratorError4 = true;
                _iteratorError4 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion4 && _iterator4.return) {
                        _iterator4.return();
                    }
                } finally {
                    if (_didIteratorError4) {
                        throw _iteratorError4;
                    }
                }
            }

            return this;
        }
    }, {
        key: 'limit',
        value: function limit(_limit, offset) {
            this.queryBuilder.limit(_limit, offset);

            return this;
        }
    }, {
        key: 'orderByDesc',
        value: function orderByDesc(column) {
            return this.orderBy('-' + column);
        }
    }, {
        key: 'where',
        value: function where(key, operator) {
            var value = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;

            this.queryBuilder.where(key, operator, value);

            return this;
        }
    }, {
        key: 'from',
        value: function from(date) {
            this.queryBuilder.from(date);

            return this;
        }
    }, {
        key: 'to',
        value: function to(date) {
            this.queryBuilder.to(date);

            return this;
        }
    }, {
        key: 'select',
        value: function select() {
            var _queryBuilder2;

            (_queryBuilder2 = this.queryBuilder).select.apply(_queryBuilder2, arguments);

            return this;
        }
    }, {
        key: 'distinct',
        value: function distinct() {
            var bool = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;

            this.queryBuilder.distinct(bool);

            return this;
        }

        // build model

        /**
         * Make changes to the received json before the parsing
         * @param json the json received
         * @returns the computed json
         */

    }, {
        key: 'computeJsonBeforeParsing',
        value: function computeJsonBeforeParsing(json) {
            return json;
        }
    }, {
        key: 'respond',
        value: function respond(json) {
            json = this.computeJsonBeforeParsing(json);
            return _ResponseHandler2.default.ofJson(this, json);
        }

        // lib

    }, {
        key: 'resourceUrl',
        value: function resourceUrl() {
            return this.getBaseUrl() + '/' + this.getNamespace() + '/';
        }
    }, {
        key: 'isCollection',
        value: function isCollection(data) {
            return _lodash2.default.isArray(data.data);
        }
    }, {
        key: 'selfValidate',
        value: function selfValidate() {
            var name = this.getNamespace();

            if (name === null || !_lodash2.default.isString(name) || name.length === 0) {
                throw new Error('LRA Query Builder: Resource name not defined in ' + this.constructor.name + ' model. Implement resourceName method in the ' + this.constructor.name + ' model to resolve this error.');
            }
        }
    }]);

    return Model;
}();

exports.default = Model;