'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.QUERY_AWAIT_COLLECTION = exports.QUERY_AWAIT_SINGLE = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _config = require('./config');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var QUERY_AWAIT_SINGLE = exports.QUERY_AWAIT_SINGLE = 'QUERY_AWAIT_SINGLE';
var QUERY_AWAIT_COLLECTION = exports.QUERY_AWAIT_COLLECTION = 'QUERY_AWAIT_COLLECTION';

var QueryBuilder = function () {
    function QueryBuilder() {
        _classCallCheck(this, QueryBuilder);

        this.reset();
    }

    _createClass(QueryBuilder, [{
        key: 'reset',
        value: function reset() {
            this.url = '';
            this.path = '';
            this.query = '';
            this.relations = [];
            this.sorts = [];
            this.fields = [];
            this.limitRows = null;
            this.fromDate = null;
            this.toDate = null;
            this.selectDistinct = false;
            this.wheres = [];
            this.owner = null;
            this.customs = [];
            this.awaitType = QUERY_AWAIT_COLLECTION;
        }
    }, {
        key: 'relationOf',
        value: function relationOf(namespace, id) {
            this.owner = {
                namespace: namespace,
                id: id
            };
        }
    }, {
        key: 'with',
        value: function _with() {
            for (var _len = arguments.length, resourceName = Array(_len), _key = 0; _key < _len; _key++) {
                resourceName[_key] = arguments[_key];
            }

            if (!this.relations[resourceName]) {
                var _relations;

                (_relations = this.relations).push.apply(_relations, resourceName);
            }
        }
    }, {
        key: 'limit',
        value: function limit(_limit, offset) {
            this.limitRows = { limit: _limit, offset: offset };
        }
    }, {
        key: 'paginate',
        value: function paginate() {
            var perPage = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 10;
            var pageNumber = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 1;

            this.limit(perPage, pageNumber * perPage - perPage);
        }
    }, {
        key: 'orderBy',
        value: function orderBy(column) {
            var direction = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'asc';

            direction = direction.toLowerCase();
            if (!['asc', 'desc'].includes(direction)) {
                throw new Error('LRA Query Builder: Invalid sort direction: "' + direction + '". Allowed only "asc" or "desc" (case insensitive).');
            }

            this.sorts.push({ column: column, direction: direction });
        }
    }, {
        key: 'where',
        value: function where(key, operator, value) {
            if (_lodash2.default.isNull(value)) {
                value = operator;
                operator = '=';
            }

            this.wheres.push({ key: key, operator: operator, value: value });
        }
    }, {
        key: 'select',
        value: function select() {
            for (var _len2 = arguments.length, fields = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
                fields[_key2] = arguments[_key2];
            }

            this.fields = fields;
        }
    }, {
        key: 'distinct',
        value: function distinct() {
            var bool = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;

            this.selectDistinct = bool;
        }
    }, {
        key: 'from',
        value: function from(date) {
            this.fromDate = date;
        }
    }, {
        key: 'to',
        value: function to(date) {
            this.toDate = date;
        }
    }, {
        key: 'addCustomParameter',
        value: function addCustomParameter(key, value) {
            this.customs.push({ key: key, value: value });
        }
    }, {
        key: 'buildUrl',
        value: function buildUrl(namespace, id) {
            this._setPath(namespace, id);

            this._appendQuery();

            return this.path + this.query;
        }
    }, {
        key: '_setPath',
        value: function _setPath(namespace, id) {
            this.path = _config.REST_CONFIG.base_url;
            this._appendOwner();
            this._appendNamespace(namespace, id);
        }
    }, {
        key: '_appendOwner',
        value: function _appendOwner() {
            if (this.owner && this.owner.namespace && this.owner.id) {
                this.path += '/' + this.owner.namespace + '/' + this.owner.id;
            }
        }
    }, {
        key: '_appendNamespace',
        value: function _appendNamespace(namespace, id) {
            this.path += '/' + namespace;
            if (id) {
                this.path += '/' + id;
            }
        }
    }, {
        key: '_appendDistinct',
        value: function _appendDistinct() {
            if (this.selectDistinct) {
                this._append(_config.REST_CONFIG.request_keywords.selectDistinct + '=' + this.selectDistinct);
            }
        }
    }, {
        key: '_appendFromTo',
        value: function _appendFromTo() {
            if (this.fromDate) {
                this._append(_config.REST_CONFIG.request_keywords.from + '=' + this.fromDate);
            }

            if (this.toDate) {
                this._append(_config.REST_CONFIG.request_keywords.to + '=' + this.toDate);
            }
        }
    }, {
        key: '_appendWheres',
        value: function _appendWheres() {
            var _iteratorNormalCompletion = true;
            var _didIteratorError = false;
            var _iteratorError = undefined;

            try {
                for (var _iterator = this.wheres[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                    var where = _step.value;

                    this._append(_config.REST_CONFIG.request_keywords.where + '[]=' + where.key + ',' + where.operator + ',' + where.value);
                }
            } catch (err) {
                _didIteratorError = true;
                _iteratorError = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion && _iterator.return) {
                        _iterator.return();
                    }
                } finally {
                    if (_didIteratorError) {
                        throw _iteratorError;
                    }
                }
            }
        }
    }, {
        key: '_appendIncludes',
        value: function _appendIncludes() {
            if (this.relations.length) {
                this._append(_config.REST_CONFIG.request_keywords.load_relations + '=' + this.relations.join(';'));
            }
        }
    }, {
        key: '_appendFields',
        value: function _appendFields() {
            if (this.fields.length) {
                this._append(_config.REST_CONFIG.request_keywords.select_fields + '=' + this.fields.join(','));
            }
        }
    }, {
        key: '_appendSort',
        value: function _appendSort() {
            if (this.sorts.length) {
                var fieldsArray = this.sorts.map(function (_ref) {
                    var column = _ref.column,
                        direction = _ref.direction;
                    return (direction === 'desc' ? '-' : '') + column;
                });
                this._append(_config.REST_CONFIG.request_keywords.order_by + '=' + fieldsArray.join(','));
            }
        }
    }, {
        key: '_appendLimit',
        value: function _appendLimit() {
            if (this.limitRows) {
                var _limitRows = this.limitRows,
                    limit = _limitRows.limit,
                    offset = _limitRows.offset;


                this._append(_config.REST_CONFIG.request_keywords.limit + '=' + limit);
                if (offset) {
                    this._append(_config.REST_CONFIG.request_keywords.offset + '=' + offset);
                }
            }
        }
    }, {
        key: '_appendCustoms',
        value: function _appendCustoms() {
            var _iteratorNormalCompletion2 = true;
            var _didIteratorError2 = false;
            var _iteratorError2 = undefined;

            try {
                for (var _iterator2 = this.customs[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
                    var _step2$value = _step2.value,
                        key = _step2$value.key,
                        value = _step2$value.value;

                    this._append(key + '=' + value);
                }
            } catch (err) {
                _didIteratorError2 = true;
                _iteratorError2 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion2 && _iterator2.return) {
                        _iterator2.return();
                    }
                } finally {
                    if (_didIteratorError2) {
                        throw _iteratorError2;
                    }
                }
            }
        }
    }, {
        key: '_appendQuery',
        value: function _appendQuery() {
            this._appendIncludes();
            this._appendLimit();
            this._appendSort();
            this._appendFromTo();
            this._appendFields();
            this._appendWheres();
            this._appendDistinct();
            this._appendCustoms();

            if (this.query.length) {
                this.query = '?' + this.query;
            }
        }
    }, {
        key: '_append',
        value: function _append(append) {
            if (this.query.length) {
                append = '&' + append;
            }

            this.query += append;
        }
    }]);

    return QueryBuilder;
}();

exports.default = QueryBuilder;