'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _jsJoda = require('js-joda');

var _QueryBuilder = require('./QueryBuilder');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var ResponseHandler = function () {
    function ResponseHandler(model, json) {
        _classCallCheck(this, ResponseHandler);

        this.model = model;
        this.json = json;
    }

    _createClass(ResponseHandler, [{
        key: 'handle',
        value: function handle() {
            if (_lodash2.default.isEmpty(this.json)) {
                return this.model.queryBuilder.awaitType == _QueryBuilder.QUERY_AWAIT_SINGLE ? {} : [];
            }

            if (_lodash2.default.isArray(this.json)) {
                return this.deserializeArray(this.json, this.model);
            }

            return this.deserializeOne(this.json, this.model);
        }
    }, {
        key: 'deserializeOne',
        value: function deserializeOne(json, model) {
            // console.log('json', json);
            var instance = this.newInstanceOf(model);
            var dates = instance.getDates();
            var dateNames = Object.keys(dates);
            var fields = instance.getFields();
            var relations = instance.getRelations();
            var relationNames = Object.keys(relations);

            for (var prop in json) {
                if (dateNames.includes(prop)) {
                    instance[prop] = this.dateFromString(dates[prop], json[prop]);
                } else if (fields.includes(prop)) {
                    instance[prop] = json[prop];
                } else if (relationNames.includes(prop)) {
                    // Allow future lazy loading
                    instance.relations[prop].loaded = true;

                    if (relations[prop].list) {
                        instance[prop] = this.deserializeArray(json[prop], new relations[prop].class());
                    } else {
                        instance[prop] = this.deserializeOne(json[prop], new relations[prop].class());
                    }
                } else if (prop === 'pivot') {
                    instance[prop] = json[prop];
                }
            }

            return instance;
        }
    }, {
        key: 'deserializeArray',
        value: function deserializeArray(jsonArray, model) {
            var _this = this;

            return jsonArray.map(function (json) {
                return _this.deserializeOne(json, model);
            });
        }
    }, {
        key: 'newInstanceOf',
        value: function newInstanceOf(obj) {
            return _lodash2.default.clone(obj);
        }
    }, {
        key: 'dateFromString',
        value: function dateFromString(dateType, dateString) {

            if (dateString === null) {
                return null;
            } else if (dateType === 'datetime') {
                dateString = dateString.replace('T', ' ');
                dateString = dateString.replace('Z', '');

                var _dateString$split = dateString.split(' '),
                    _dateString$split2 = _slicedToArray(_dateString$split, 2),
                    dateStr = _dateString$split2[0],
                    timeStr = _dateString$split2[1];

                var _dateStr$split$map = dateStr.split('-').map(Number),
                    _dateStr$split$map2 = _slicedToArray(_dateStr$split$map, 3),
                    y = _dateStr$split$map2[0],
                    m = _dateStr$split$map2[1],
                    d = _dateStr$split$map2[2];

                var _timeStr$split$map = timeStr.split(':').map(Number),
                    _timeStr$split$map2 = _slicedToArray(_timeStr$split$map, 3),
                    h = _timeStr$split$map2[0],
                    i = _timeStr$split$map2[1],
                    s = _timeStr$split$map2[2];

                return _jsJoda.LocalDateTime.of(y, m, d, h, i, s);
            } else if (dateType === 'time') {
                var _dateString$split$map = dateString.split(':').map(Number),
                    _dateString$split$map2 = _slicedToArray(_dateString$split$map, 3),
                    _h = _dateString$split$map2[0],
                    _i = _dateString$split$map2[1],
                    _s = _dateString$split$map2[2];

                return _jsJoda.LocalTime.of(_h, _i, _s);
            } else if (dateType === 'date') {
                var _dateString$split$map3 = dateString.split('-').map(Number),
                    _dateString$split$map4 = _slicedToArray(_dateString$split$map3, 3),
                    _y = _dateString$split$map4[0],
                    _m = _dateString$split$map4[1],
                    _d = _dateString$split$map4[2];

                return _jsJoda.LocalDate.of(_y, _m, _d);
            }

            return null;
        }
    }], [{
        key: 'ofJson',
        value: function ofJson(model, json) {
            var handler = new ResponseHandler(model, json);
            return handler.handle();
        }
    }]);

    return ResponseHandler;
}();

exports.default = ResponseHandler;